import { Injectable } from '@angular/core';
import { APIServer } from './apiserver';
import {BehaviorSubject, pipe, Observable, combineLatest, of} from 'rxjs';
import {map, tap, switchMap, timeout, catchError } from 'rxjs/operators';

import { HttpClientModule, HttpClient, HttpHeaders, HttpErrorResponse, HttpResponse } from '@angular/common/http';

@Injectable({
  providedIn: 'root'
})
export class BackendSelectionService {

  public apiservers: BehaviorSubject<APIServer[]>;
  public apiserver: BehaviorSubject<APIServer>;
  public localapi: BehaviorSubject<boolean>;

    constructor(private http: HttpClient) { 
        this.apiservers = new BehaviorSubject([]);
        this.apiserver = new BehaviorSubject(null);
        this.localapi = new BehaviorSubject(false);
        this.initApiServer();
        this.getAPIServers();
        this.apiserver.subscribe((v) => console.log('apiserver change',v));
        this.apiservers.subscribe((v) => console.log('apiservers change',v));
    }


  private testApiServers(servers: APIServer[]): Observable<any> {
    var obs: Observable<any>[] = [];
    let headers = new HttpHeaders();
    let options = { headers: headers, withCredentials: false};
    console.log('entering testAPIServers');
    for (let server of servers) {
      //obs.push( this.http.get(s.tes+'/sshagent',options).pipe(map((r) => {return {'server':s,'check':r}})));
      obs.push( this.http.get<string>(server.tes+'/sshagent',options).pipe(
        timeout(2000),
        catchError(e=> {console.error('testing api server',server); console.error(e); return of(null)}),
        map((v) => {return {'server':server,'query':v}}),
        ));
    }
    return combineLatest(obs);
  }
 
    private testApiServer(server: APIServer) {
      this.http.get(server.tes+'/sshagent').subscribe((ok) => {return true}, (err) => {return false})
    }

    private initApiServer() {
        try {
            this.apiserver.next(<APIServer>JSON.parse(localStorage.getItem('apiserver')));
        } catch {
          this.apiserver.next(null);
        }
      if (this.apiserver.value === null) {
          this.apiservers.subscribe((v) => this.defaultApiServer(v))
      }
      if (this.apiserver.value !== undefined && this.apiserver.value !== null && this.apiserver.value.tes.indexOf('localhost') != -1) {
        this.localapi.next(true);
        console.log('localapi true', this.apiserver.value.tes);
      } else {
        this.localapi.next(false);
      }
    }

    private defaultApiServer(v: APIServer[]) {
      this.apiserver.next(v[0]);
    }

    private saveLastApiServer(s: APIServer) {
        localStorage.setItem('apiserver', JSON.stringify(s));
    }

     setApiServer(server: APIServer) {
        this.apiserver.next(server);
      if (this.apiserver.value.tes.indexOf('localhost') != -1) {
        this.localapi.next(true);
        console.log('localapi true', this.apiserver.value.tes);
      } else {
        this.localapi.next(false);
        console.log('localapi false',  this.apiserver.value.tes);
      }

        this.saveLastApiServer(this.apiserver.value)
     }

    /*     storeLocalAPIServers(apiservers) {
         localStorage.setItem('localAPIServers',JSON.stringify(apiservers));
         this.getAPIServers();
     }
     
     removeLocalAPIServer() {
         localStorage.removeItem('localAPIServers');
         this.getAPIServers();
     }*/

     getAPIServers() {
       let headers = new HttpHeaders();
       let options = { headers: headers, withCredentials: false};
       this.http.get<APIServer[]>('./assets/config/apiservers.json',options).pipe(
         switchMap((v) => this.testApiServers(v)),
         tap((v) => console.log('testapiservers observable fired',v)),
         map((list, idx) => (<any[]>list)
                              .filter(function (v) { return v.query != null})
                              .map(function (v) {return v.server})) // Note the two maps are not the same one is an rxjs observable map one is a list map
       ).subscribe(resp => this.updateAPIServers(resp));
     }

     private updateAPIServers(resp) {
         var s: APIServer;
         var list: APIServer[] = []
         var current: APIServer;
         var found: boolean; 
         var localServers: APIServer[] = [];
         current = this.apiserver.value;
         found = false;
         try {
             localServers = <APIServer[]>JSON.parse(localStorage.getItem('localAPIServers'));
         } catch {
             localServers = []
         }

         for (s of <APIServer[]>resp) {
             if (current !== undefined && current !== null && s.name == current.name) {
                 list.push(current);
                 found = true;
             } else {
                 list.push(s);
             }
         }
         if (localServers !== null) {
             for (s of localServers) {
                 if (current !== null && s.name == current.name) {
                     list.push(current);
                     found = true;
                 } else {
                     list.push(s);
                 }
             } 
         }
         if (!found && current !== undefined) {
             list.push(current);
         }
         this.apiservers.next(list);
     }

}
